from enum import Enum

from colorama import Fore

class RuleCategory(Enum):
    NAME_HAS_DASH       = ("DASH_IN_NAME", Fore.LIGHTMAGENTA_EX)
    NAME_IS_RESOURCE    = ("NAME_IS_RESOURCE", Fore.CYAN)
    NAME_UNIQUENESS     = ("NAME_UNIQUENESS", Fore.BLUE)
    OUTPUT_EXPOSURE     = ("OUTPUT_EXPOSURE", Fore.YELLOW)
    FILE_PURPOSE        = ("FILE_PURPOSE", Fore.GREEN)
    MISSING_FILE        = ("MISSING_FILE", Fore.RED)
    PROVIDER_DEFINITION = ("PROVIDER_DEFINITION", Fore.LIGHTRED_EX)
    PROVIDER_ALIAS      = ("PROVIDER_ALIAS", Fore.LIGHTBLUE_EX)
    FILENAME_CONSTRAINT = ("FILENAME_CONSTRAINT", Fore.LIGHTGREEN_EX)